import { JsonPath, SyncCrawlHook } from "json-crawl";
import { ComapreContext, CompareRule, CompareRules } from "./rules";
import { ClassifierType, DiffAction } from "../core/constants";
export type ActionType = keyof typeof DiffAction;
export type DiffType = typeof ClassifierType[keyof typeof ClassifierType];
export type Diff = {
    type: DiffType;
    action: ActionType;
    path: JsonPath;
    before?: any;
    after?: any;
    description?: string;
};
export type DiffMeta = {
    action: ActionType;
    type: DiffType;
    replaced?: any;
};
export interface CompareResult {
    diffs: Diff[];
    merged: any;
    rootMergeMeta?: MergeMeta;
}
export type MergeMeta = DiffMeta | MergeArrayMeta;
export type MergeArrayMeta = {
    array: Record<number, MergeMeta>;
};
export type SourceContext = {
    before?: {
        source: unknown;
        jsonPath: JsonPath;
    };
    after?: {
        source: unknown;
        jsonPath: JsonPath;
    };
};
export type ComapreOptions = {
    rules?: CompareRules;
    metaKey?: string | symbol;
    arrayMeta?: boolean;
    annotateHook?: AnnotateHook;
    externalSources?: {
        before?: Record<string, unknown>;
        after?: Record<string, unknown>;
    };
};
export type CompareEngine = (before: unknown, after: unknown, options?: ComapreOptions, context?: SourceContext) => CompareResult;
export type AnnotateHook = (diff: Diff, ctx: ComapreContext) => string;
export type NodeRoot = {
    "#": any;
};
export type KeyMapping = Record<string | number, string | number>;
export interface MergeState<T extends string | number = string> {
    keyMap: KeyMapping;
    aPath: JsonPath;
    aNode: JsonNode<T>;
    bPath: JsonPath;
    bNode: JsonNode<T>;
    mNode: any;
    parentMeta: MergeMetaRecord;
    root: {
        before: NodeRoot;
        after: NodeRoot;
        merged: JsonNode<T>;
    };
}
export type MergeMetaRecord = Record<string | number, MergeMeta>;
export type JsonNode<T extends string | number = string> = T extends string ? Record<string | number, unknown> : Array<unknown>;
export interface DiffFactory {
    added: (path: JsonPath, after: unknown, ctx: ComapreContext) => Diff;
    removed: (path: JsonPath, before: unknown, ctx: ComapreContext) => Diff;
    replaced: (path: JsonPath, before: unknown, after: unknown, ctx: ComapreContext) => Diff;
    renamed: (path: JsonPath, before: unknown, after: unknown, ctx: ComapreContext) => Diff;
}
export interface MergeFactoryResult {
    diffs: Diff[];
    hook: SyncCrawlHook<MergeState, CompareRule>;
}
export interface ContextInput extends MergeState {
    before: any;
    after: any;
    bPath: JsonPath;
    akey: string | number;
    bkey: string | number;
    rules: CompareRules;
}
